using System;
using System.Collections.Generic;
using System.Linq;

namespace LinqExamples.Extensions
{
  public static class Extensions
  {
    public static void WriteLineAll<T>(this IEnumerable<T> items, string title)
    {
      items.WriteAll(title, Environment.NewLine);
    }

    public static void WriteAll<T>(this IEnumerable<T> items, string title, string separator = ", ")
    {
      WriteTitle(title);
      foreach (var n in items)
      {
        Console.Write(n + separator);
      }
    }

    private static void WriteTitle(string title)
    {
      Console.WriteLine();
      if (!string.IsNullOrEmpty(title))
      {
        Console.WriteLine(title);
      }

      Console.WriteLine(string.Join(string.Empty,
        Enumerable.Repeat("=", GetTitleLength(title))));
    }

    private static int GetTitleLength(string title)
    {
      const int defaultLength = 3;
      if (title == null)
      {
        return defaultLength;
      }
      return Math.Max(title.Length, defaultLength);
    }
  }
}