using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using LinqExamples.Data;
using LinqExamples.Extensions;

namespace LinqExamples
{
  internal partial class LinqOperatorExamples
  {
    private void OrderingExamples()
    {
      var products = _dataContext.GetProducts();
      var orderedProducts = products.OrderBy(c => c.UnitsInStock);

      WriteFirst7Products(orderedProducts, "OrderBy UnitsInStock");
      WriteFirst7Products(orderedProducts.OrderBy(c => c.ProductName), "OrderBy UnitsInStock, OrderBy ProductName");
      WriteFirst7Products(orderedProducts.ThenBy(c => c.ProductName), "OrderBy UnitsInStock, ThenBy ProductName");

      //var descending = 
      //  from p in products
      //  orderby p.UnitsInStock descending
      //  select p;

      //WriteFirst7Products(descending, "OrderBy UnitsInStock descending");
      //WriteFirst7Products(descending.ThenByDescending(c => c.ProductName), "OrderBy UnitsInStock descending, ThenBy ProductName descending");

    }

    private static void WriteFirst7Products(IEnumerable<Product> products, string title)
    {
      products.Take(7).Select(c => c.ProductName + " " + c.UnitsInStock).WriteLineAll(title);
    }

    private void GroupingExamples()
    {
      var products = _dataContext.GetProducts();
      var groupByCategory = products.GroupBy(c => c.Category);

      foreach (var group in groupByCategory)
      {
        @group.Select(c => c.ProductName).WriteLineAll("Products in category " + @group.Key);
      }

      //var groupByFirstLetter = from p in products
      //  group p by p.ProductName[0] into g
      //  select g;

      //foreach (var group in groupByFirstLetter)
      //{
      //  group.Select(c => c.ProductName).WriteLineAll("Products with first letter " + group.Key);
      //}      
    }
  }
}