using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using LinqExamples.Data;
using LinqExamples.Extensions;

namespace LinqExamples
{
  internal partial class LinqOperatorExamples
  {
    private void JoiningExamples()
    {
      List<Customer> customers = _dataContext.GetCustomers();
      List<Supplier> suppliers = _dataContext.GetSuppliers();

      GroupJoin(suppliers, customers);
      //Join(suppliers, customers);
      //Zip();
    }

    private static void Join(List<Supplier> suppliers, List<Customer> customers)
    {
      var custSupJoin =
        from sup in suppliers
        join cust in customers on sup.Country equals cust.Country
        select new { Country = sup.Country, SupplierName = sup.SupplierName, CustomerName = cust.CompanyName };

      var fluentAlternative = suppliers.Join(customers, sup => sup.Country, cust => cust.Country,
        (sup, cust) => new { Country = sup.Country, SupplierName = sup.SupplierName, CustomerName = cust.CompanyName });


      custSupJoin.Select(c => string.Format("Country={0},SN={1},CN={2}", c.Country, c.SupplierName, c.CustomerName))
        .WriteLineAll("Suppliers join customers on equals country");
    }

    private static void GroupJoin(List<Supplier> suppliers, List<Customer> customers)
    {
      var custSupQuery =
        from sup in suppliers
        join cust in customers on sup.Country equals cust.Country into cs
        select new { Key = sup.Country, Items = cs };

      var custSupQueryFluent =
        suppliers.GroupJoin(customers,
          sup => sup.Country,
          cust => cust.Country,
          (sup, cs) => new { Key = sup.Country, Items = cs });

      foreach (var group in custSupQuery)
      {
        @group.Items.WriteLineAll(@group.Key);
      }
    }

    private void Zip()
    {
      var evenNumbers = new[] { 1, 3, 5, 7, 9 };
      var oddNumbers = new[] { 2, 4, 6, 8 };
      evenNumbers.Zip(oddNumbers, (i, i1) => i * i1).WriteAll("Zip");
    }

  }
}