using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Xml;
using System.Xml.Linq;
using LinqExamples.Data;
using LinqExamples.Extensions;

namespace LinqExamples
{
  internal partial class LinqOperatorExamples
  {
    private void AggregationExamples()
    {
      int[] numbers = { 5, 4, 5, 1, 3, 9, 8, 6, 6, 5, 7 };
      Console.WriteLine("Min number:" + numbers.Min());
      Console.WriteLine("Max number:" + numbers.Max());
      Console.WriteLine("Sum of numbers:" + numbers.Sum());
      Console.WriteLine("Average of numbers:" + numbers.Average());

      Console.WriteLine("Max odd number:" + numbers.Max(n => n % 2 == 0));

      var products = _dataContext.GetProducts();
      Console.WriteLine("Average units in stock:" + products.Average(p => p.UnitsInStock));
      Console.WriteLine("Max unit price:" + products.Average(p => p.UnitPrice));

      var productsWithLowerPriceThanAverage = products.Where(p => p.UnitPrice < products.Average(pr => pr.UnitPrice));
      productsWithLowerPriceThanAverage.Select(p => p.ProductName).WriteLineAll("Products with price lower than average price");

      var productWithHighestValue = products.Aggregate((max, next) => max == null || max.UnitsInStock * max.UnitPrice < next.UnitsInStock * next.UnitPrice ? next : max);
      Console.WriteLine();
      Console.WriteLine("Product with highest units in stock * unit price:" + productWithHighestValue.ProductName);
    }

    private void QualifiersExamles()
    {
      int[] numbers = { 5, 4, 5, 1, 3, 9, 8, 6, 6, 5, 7 };

      if (numbers.Contains(6))
      {
        Console.WriteLine("Numbers contains 6");
      }

      if (numbers.All(n => n < 10))
      {
        Console.WriteLine("All numbers are less than 10");
      }

      if (numbers.Any(n => n % 2 == 0))
      {
        Console.WriteLine("Numbers contains at least one odd number.");
      }

      int[] numbers2 = { 5, 4, 5, 1, 3, 9, 8, 6, 6, 5, 7 };
      if (numbers.SequenceEqual(numbers2))
      {
        Console.WriteLine("Numbers and numbers2 are equal.");
      }
    }

    private void ElementOperators()
    {
      int[] numbers = { 5, 4, 5, 1, 3, 9, 8, 6, 6, 5, 7 };
      numbers.WriteAll("Numbers");

      Console.WriteLine("First " + numbers.First());
      Console.WriteLine("First " + numbers.First(c => c > 100)); // Exception
      Console.WriteLine("First or default " + numbers.FirstOrDefault(c => c > 100));
      Console.WriteLine("Last " + numbers.Last());
      Console.WriteLine("Last " + numbers.Last(c => c > 100)); // Exception
      Console.WriteLine("Last or default" + numbers.LastOrDefault(c => c > 100));

      Console.WriteLine("Single " + numbers.Single()); // Exception
      Console.WriteLine("Single " + numbers.Single(c => c % 2 == 0)); // Exception
      Console.WriteLine("Single " + numbers.Single(c => c == 7));
      Console.WriteLine("Single or default" + numbers.SingleOrDefault(c => c > 100));

      Console.WriteLine("ElementAt " + numbers.ElementAt(2));
      Console.WriteLine("ElementAt " + numbers.ElementAt(100)); // Exception
      Console.WriteLine("ElementAtOrDefault " + numbers.ElementAt(100));

      new List<int>().DefaultIfEmpty().WriteLineAll("DefaultIfEmpty");
    }

    private void GenerationExamplas()
    {
      int[][] numbers =
      {
        new int[] {1, 2, 3},
        new int[] {4, 5, 6},
        null
      };

      IEnumerable<int> flat = numbers.SelectMany(innerArray => innerArray ?? Enumerable.Empty<int>());

      var repeatCollection = Enumerable.Repeat("repeat", 5);
      repeatCollection.WriteAll("Repeat repeat 5 times");

      var rangeCollection = Enumerable.Range(0, 10);
      rangeCollection.WriteAll("Range from 0 to 9");
    }
  }
}