using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using LinqExamples.Data;
using LinqExamples.Extensions;

namespace LinqExamples
{
  internal partial class LinqOperatorExamples
  {

    private void ProjectionExamples()
    {
      SelectWithAnonymousType();
      //SelectVsSelectMany();
      //SelectManyInQuerySyntax();
    }

    private void SelectWithAnonymousType()
    {
      var products = _dataContext.GetProducts();
      var productValues = products.Select(c => new { Value = c.UnitsInStock * c.UnitPrice, c.ProductName });
      foreach (var item in productValues)
      {
        Console.WriteLine("{0} - {1}", item.ProductName, item.Value);
      }
    }

    private static void SelectVsSelectMany()
    {
      var fullNames = new[] { "Anne Williams", "John Fred Smith", "Sue Green" }.AsQueryable();
      IEnumerable<string[]> query1 = fullNames.Select(name => name.Split());
      IEnumerable<string> query2 = fullNames.SelectMany(name => name.Split());

      query1.WriteLineAll("A simple Select gives us a hierarchical result set");
      query2.WriteLineAll("SelectMany gives us a flat result set");

      IEnumerable<string> query3 =
        from fullName in fullNames
        from name in fullName.Split() // Translates to SelectMany
        select name;

      query3.WriteLineAll("Select many in query expression syntax:");
    }

    private void SelectManyInQuerySyntax()
    {
      var fullNames = new[] { "Anne Williams", "John Fred Smith", "Sue Green" }.AsQueryable();
      IEnumerable<string> query4 =
        from fullName in fullNames
        from name in fullName.Split()
        select name + " is from " + fullName;

      query4.WriteLineAll("SelectMany with outer variable:");

      IEnumerable<string> query5 =
        fullNames.SelectMany(fullName => fullName.Split().Select(name => new { name, fullName }))
          .Select(x => x.name + " is from " + x.fullName);

      IEnumerable<string> query6 =
        fullNames.SelectMany(fullName => fullName.Split(), (fullName, name) => name + " is from " + fullName);

    }    
  }
}